#========================================================================#
# Script Name: mp_timeseries.py                                          #
#                                                                        #
# Description: This script defines functions that will allow users to    #
#              choose atlases to use to define ROIs and generate         #
#              timeseries                                                #
#                                                                        #
# Author:      Jen Burrell (March 21st, 2023)                            #
#========================================================================#
import os.path
import subprocess
import glob
import PySimpleGUI as sg
from misc import *
from itertools import repeat
import numpy as np
from multiprocessing import Pool

SDIR = os.path.dirname(__file__) # find where script is housed
#---------------------------------------#
#       Get data from chosen atlas      #
#---------------------------------------#
def get_atlas(atlas, WDIR, func):
    a = {'dir': (f"{SDIR}/Atlases/{atlas}")}
    pix_info = ''.join(str(subprocess.check_output(["fslinfo", f"{func}"])).split('\\t')).split('\\')
    pix_idx = [i for i, s in enumerate(pix_info) if 'npixdim1' in s]
    pix_dim = pix_info[pix_idx[0]].split("npixdim1")[1].split('.')[0]
    if atlas == "Brainnetome":
        a['lut'] = os.path.join(a['dir'], f"{atlas}.lut")
        a['file'] = os.path.join(a['dir'], f"BN_Atlas_246_{pix_dim}mm.nii.gz")
        a['file'] = os.path.basename(a['file'].strip())
        a['root'] = atlas
        min, a['max']  = subprocess.getoutput([f"fslstats {a['dir']}/{a['file']} -R"]).split()
    elif atlas == "Schaefer":
        layout = [
            [sg.Text("Select file for Schaefer parcellations:"), sg.Input(key='-File-', enable_events=True), sg.FilesBrowse('Select', key='-IN2-')],
            [sg.Button("Ok"),sg.Button("Cancel")]]
        window = sg.Window('Schaefer Parcellations', layout, resizable=True, enable_close_attempted_event=True)
        window["-IN2-"].InitialFolder = a['dir']
        while True:
            event, values = window.read()
            if event == sg.WIN_CLOSED:
                break
            if event == '-WINDOW CLOSE ATTEMPTED-':
                exit("User exited ConCorr")
            if event in 'Cancel':
                sg.popup_no_border("No file selected, please try again")
            if event == '-File-':
                a['file'] = values['-File-']
                window.close()
            if event == 'Ok':
                if 'file' in a and file_exists(a['file']):
                    window.close()
                else:
                    sg.popup_no_border("No file selected, please try again")
        window.close()
        a['file'] = a['file'].split('/')[-1]
        a['root'] = a['file'].split('_FSL')[0]
        a['lut'] = ''.join(glob.glob(os.path.join(a['dir'], "fsleyes_lut", f"{a['root']}*")))
        min, a['max']  = subprocess.getoutput([f"fslstats {a['dir']}/{a['file']} -R"]).split()
    elif atlas == "HCP-MMP1":
        a['lut'] = os.path.join(a['dir'], "HCP-MMP1_UniqueRegionList.txt")
        a['root'] = atlas
        a['file'] = "MMP_in_MNI_corr.nii.gz"
        min, a['max']  = subprocess.getoutput([f"fslstats {a['dir']}/{a['file']} -R"]).split()
    elif atlas == "Juelich":
        a['lut'] = os.path.join(a['dir'], "Juelich_atlas_index.txt")
        a['root'] = atlas
        a['file'] = "Juelich-maxprob-thr25-2mm.nii.gz"
        min, a['max']  = subprocess.getoutput([f"fslstats {a['dir']}/{a['file']} -R"]).split()
    elif atlas == "HMAT":
        a['lut'] = os.path.join(a['dir'], "HMAT_atlas_index.txt")
        a['root'] = atlas
        a['file'] = "HMAT.nii"
        min, a['max']  = subprocess.getoutput([f"fslstats {a['dir']}/{a['file']} -R"]).split()
    elif atlas == "Provided_ROIs":
        a['lut'] = ''
        a['root'] = atlas
        a['file'] = os.listdir(f"{WDIR}/ROIs")
        a['max'] = len(a['file'])
        a['dir'] = (f"{WDIR}/ROIs")
    else:
        raise ValueError(f"Unknown atlas {atlas}")
    return a
#---------------------------------------#
#           Mask Creation and           #
#         Timeseries generation         #
#---------------------------------------#
def run_timeseries(*dicts):
    user_info = {}
    for dict in dicts:
        user_info.update(dict)
    print(user_info)
    
    func_data = user_info['func_data']
    
    # - get path for all participants - #
    for char in func_data:
        if char.isdigit():
            func_data = func_data.replace(char, '*')
            
    dir_path_list = sorted(glob.glob(os.path.dirname(func_data))) # list of all files
    
    # - multiprocessing - #
    pool = Pool() # Create a multiprocessing Pool
    pool.starmap(get_masks, zip(dir_path_list, repeat(user_info))) # process gets masks
    pool.starmap(add_timeseries, zip(dir_path_list, repeat(user_info))) # process adds timeseries to master file
    # Close pool
    pool.close()
    pool.join()
    
#---------------------------------------#
#      Generate Masks & Timeseries      #
#---------------------------------------#
def get_masks(dir_path, user_info):
    # - define variables in use - #
    func_data = user_info['func_data']
    atlas = user_info['atlas'] # name of atlas
    atlas_lut_file = user_info['lut']
    atlas_directory = user_info['dir']
    parce_file = user_info['file'] # name of atlas file
    parce_name = user_info['root'] # name of atlas
    max = int(float(user_info['max']))
    if user_info['advanced'] == 'Yes':
        user_thr = user_info['thr']
    else:
        user_thr = 0.75
    
    func_file = os.path.basename(func_data)
    func_file_noext = func_file.split('.')[0] # file naming
        
    name_val = 1 # lut counting
    
    mkdir(dir_path,f"ROI_timeseries/{parce_name}")
    mkdir(dir_path,f"ROI_masks/{parce_name}")
            
    for value in range(1, int(float(max))+1):
        if atlas == "HCP-MMP1":
            atlas_lut_file = user_info['lut']
            name_val += 1  # accounts for space in data values
            roi_name = subprocess.check_output(f"sed -n '{name_val+1}p' {atlas_lut_file} | awk -F ',' '{{ print $1; }}'", shell=True).decode("utf-8").strip()  # get roi name from naming file
            if value >= 181:  # accounts for space in data values
                value += 20
        elif atlas == "Juelich":
            atlas_lut_file = user_info['lut']
            roi_name = subprocess.check_output(f"sed -n '{value+1}p' {atlas_lut_file} | awk -F '\\t' '{{ print $1; }}'", shell=True).decode("utf-8").strip()  # get roi name from naming file
            roi_name = roi_name.split(" ")[-1].replace(" ", "_")
        elif atlas == "Provided_ROIs":
            roi_name_file = os.listdir(atlas_directory)[value-1]
            roi_name = os.path.splitext(roi_name_file)[0]
            file_root = atlas
        elif atlas == "Schaefer":
            atlas_lut_file = user_info['lut']
            roi_name = subprocess.check_output(f"sed -n '{value}p' {atlas_lut_file} | awk '{{ print $5; }}'", shell=True).decode("utf-8").strip()  # get roi name from naming file
        else:
            atlas_lut_file = user_info['lut']
            roi_name = subprocess.check_output(f"sed -n '{value+1}p' {atlas_lut_file} | awk '{{ print $5; }}' | sed 's#/#_#g'", shell=True).decode("utf-8").strip()  # get roi name from naming file and remove any slashes
        ### --- output in MNI space --- ###
        if user_info['out_space'] == 'MNI space':
            if user_info['data_space'] == 'No': # check to see if data is in standard space
                ### --- Normalize subject data to MNI space --- ###
        ### - HAVE JUSTIN DOUBLE CHECK - ###
                if file_exists(f"{dir_path}/reg"):
                    subprocess.run(["flirt", "-ref", f"{dir_path}/reg/highres.nii.gz", "-in", f"{dir_path}/reg/example_func.nii.gz", "-dof", "6", "-omat", f"{dir_path}/reg/example_func2highres.mat"]) # produce affine matrix of registration of the example func to the highres file (struct)
                    subprocess.run(["flirt", "-ref", f"{FSLDIR}/data/standard/MNI152_T1_2mm_brain", "-in", f"{dir_path}/reg/highres.nii.gz", "-omat", f"{dir_path}/reg/highres2standard.mat"]) # produce affine matrix of registration of the highres file to standard space
                    subprocess.run(["convert_xfm", "-concat", f"{dir_path}/reg/example_func2highres.mat", f"{dir_path}/reg/highres2standard.mat", "-omat", f"{dir_path}/reg/example_func2standard.mat"]) # concatinates matrices to apply once
                    subprocess.run(["fnirt", f"--in={dir_path}/reg/highres_head.nii.gz", f"--aff={dir_path}/reg/highres2standard.mat", f"--cout={dir_path}/reg/highres2standard_warp.nii.gz", "--config=T1_2_MNI152_2mm"]) # produce the warp-field from registering the native space file to standard space, using parameters in T1_2_MNI152_2mm file
                    subprocess.run(["applywarp", "-i", f"{dir_path}/{func_file}", "-r", f"{dir_path}/reg/example_func.nii.gz", "-o", f"{dir_path}/{func_file_noext}_standard.nii.gz", f"--postmat={dir_path}/reg/example_func2standard.mat", f"--warp={dir_path}/reg/highres2standard_warp.nii.gz"]) # apply warp to func file such that it is in MNI space
                    func_file = f"{func_file_noext}_standard.nii.gz"
                else:
                    user_out = reg_gui()
                    user_info.update(user_out)
            ### --- generate masks --- ###
            if not file_exists(f"{dir_path}/ROI_masks/{parce_name}/{roi_name}_func.nii.gz"):
                if not file_exists(f"{dir_path}/ROI_masks/{parce_name}/{roi_name}_standard"):
                    if atlas == "Provided_ROIs":
                        subprocess.run(["cp", f"{atlas_directory}/{roi_name_file}", f"{dir_path}/ROI_masks/{parce_name}/{roi_name}_standard.nii.gz"]) # copy user's masks to folder
                    else:
                        subprocess.run(["fslmaths", f"{atlas_directory}/{parce_file}", "-thr", f"{value}", "-uthr", f"{value}", "-bin", f"{dir_path}/ROI_masks/{parce_name}/{roi_name}_standard.nii.gz"]) # find ROI that matches value and remove everything else
                if file_exists(f"{dir_path}/reg/example_func2standard.nii.gz"):
                    subprocess.run(["flirt", "-in", f"{dir_path}/ROI_masks/{parce_name}/{roi_name}_standard.nii.gz", "-ref", f"{dir_path}/reg/example_func2standard.nii.gz", "-out", f"{dir_path}/ROI_masks/{parce_name}/{roi_name}_func.nii.gz", "-usesqform", "-applyxfm"]) # aligns the NIFTI mm coordinates
                else:
                    subprocess.run(["flirt", "-in", f"{dir_path}/ROI_masks/{parce_name}/{roi_name}_standard.nii.gz", "-ref", f"{dir_path}/{func_file}", "-out", f"{dir_path}/ROI_masks/{parce_name}/{roi_name}_func.nii.gz", "-usesqform", "-applyxfm"]) # aligns the NIFTI mm coordinates
                ###### THIS LINE MIGHT NOT BE USEFUL ####
                subprocess.run(["fslmaths", f"{dir_path}/ROI_masks/{parce_name}/{roi_name}_func.nii.gz", "-thr", f"{user_thr}", "-bin", f"{dir_path}/ROI_masks/{parce_name}/{roi_name}_func.nii.gz"]) # threshold and binarize MNI mask
                os.remove(f"{dir_path}/ROI_masks/{parce_name}/{roi_name}_standard.nii.gz") # delete standard files
        ### --- output in subject space --- ###
        elif user_info['out_space'] == 'Subject space':
            if not file_exists(f"{dir_path}/ROI_masks/{parce_name}/{roi_name}_standard.nii.gz"): # Check if ROI mask has been made
                subprocess.run(["fslmaths", f"{atlas_directory}/{parce_file}", "-thr", f"{value}", "-uthr", f"{value}", "-bin", f"{dir_path}/ROI_masks/{parce_name}/{roi_name}_standard"])
                if not file_exists(f"{dir_path}/reg/highres2standard_warp_inv.nii.gz"): # check if inverse warp exists, if not, run registrations and create it
                    subprocess.run(["flirt", "-ref", f"{dir_path}/reg/highres.nii.gz", "-in", f"{dir_path}/reg/example_func.nii.gz", "-dof", "6", "-omat", f"{dir_path}/reg/example_func2highres.mat"]) # produce affine matrix of registration of the example func to the highres file (struct)
                    subprocess.run(["flirt", "-ref", f"{FSLDIR}/data/standard/MNI152_T1_2mm_brain", "-in", f"{dir_path}/reg/highres.nii.gz", "-omat", f"{dir_path}/reg/highres2standard.mat"]) # produce affine matrix of registration of the highres file to standard space
                    subprocess.run(["fnirt", f"--in={dir_path}/reg/highres_head.nii.gz", f"--aff={dir_path}/reg/highres2standard.mat", f"--cout={dir_path}/reg/highres2standard_warp.nii.gz", "--config=T1_2_MNI152_2mm"]) # produce the warp-field from registering the native space file to standard space, using parameters in T1_2_MNI152_2mm file
                    subprocess.run(["convert_xfm", "-omat", f"{dir_path}/reg/highres2example_func.mat", "-inverse", f"{dir_path}/reg/example_func2highres.mat"]) # inverts transformation such that it registers highres to example_func
                    subprocess.run(["invwarp", f"--ref={dir_path}/reg/highres_head.nii.gz", f"--warp={dir_path}/reg/highres2standard_warp.nii.gz", f"--out={dir_path}/reg/highres2standard_warp_inv"]) # inverts the warp-field such that it registers standard to native space
                subprocess.run(["applywarp", "-i", f"{dir_path}/ROI_masks/{parce_name}/{roi_name}_standard", "-r", f"{dir_path}/reg/example_func.nii.gz", "-o", f"{dir_path}/ROI_masks/{parce_name}/{roi_name}_func", f"--postmat={dir_path}/reg/highres2example_func.mat", "-w", f"{dir_path}/reg/highres2standard_warp_inv.nii.gz"]) # apply warp to ROI file such that it is in native functional space
                if not file_exists(f"{dir_path}/reg/example_func_brain_mask.nii.gz"):
                    subprocess.run(["bet", f"{dir_path}/reg/example_func.nii.gz", f"{dir_path}/reg/example_func_brain", "-f", "0.1", "-m"]) # create example functional brain mask via brain extraction
                subprocess.run(["fslmaths", f"{dir_path}/ROI_masks/{parce_name}/{roi_name}_func", "-thr", f"{user_thr}", "-bin", f"{dir_path}/ROI_masks/{parce_name}/{roi_name}_func"]) # threshold and binarize mask in native-space
                subprocess.run(["fslmaths", f"{dir_path}/ROI_masks/{parce_name}/{roi_name}_func", "-mul", f"{dir_path}/reg/example_func_brain_mask", f"{dir_path}/ROI_masks/{parce_name}/{roi_name}_func"]) # constrain mask to within the brain
        ### --- generate timeseries from masks--- ###
        if not file_exists(f"{dir_path}/ROI_timeseries/{parce_name}/{roi_name}_timeseries.txt"):
            get_timeseries(roi_name, f"{dir_path}/{func_file}", f"{dir_path}/ROI_timeseries/{parce_name}/{roi_name}_timeseries.txt", f"{dir_path}/ROI_masks/{parce_name}/{roi_name}_func.nii.gz")
            print(f"Extracting time series from {roi_name} ({value}/{max}) for {dir_path}")
 
 
#---------------------------------------#
#          Generate Timeseries          #
#---------------------------------------#
def get_timeseries(roi_name, input, out, mask, with_heading=True, print=True):
    if not file_exists(out):
        subprocess.run(["fslmeants", "-i", f"{input}", "-o", f"{out}", "-m", f"{mask}"])
    if with_heading == True:
        out_heading = ''.join([out.rsplit('.', 1)[0], '_w_heading.txt'])
        with open(out) as infile, open(out_heading, "w") as outfile:
            outfile.write(f"{roi_name}\n")
            outfile.write(infile.read())
            
#---------------------------------------#
#            Add Timeseries to          #
#             Master txt File           #
#---------------------------------------#
def add_timeseries(dir_path, user_info):
    parce_name = user_info['root'] # name of atlas
    input = sorted(glob.glob(f"{dir_path}/ROI_timeseries/{parce_name}/*_timeseries_w_heading.txt"))
    out =  f"{dir_path}/ROI_timeseries/{parce_name}_timeseries.txt"
    if not file_exists(out):
        file_dir = input[0].rsplit('/',1)[0]
        print(file_dir)
        columns =[]
        for file in input:
            with open(file, "r") as infile:
                new = np.array([line_num.rstrip() for line_num in infile.readlines()])
                columns.append(new)
        columns = np.vstack(columns).T
        np.savetxt(out, columns, fmt="%s", delimiter='\t')
    for file in input: os.remove(file)
        

if __name__ == '__main__':
    user_info = {'WDIR': '/Users/labmanager/Resilience', 'sub_count': 4, 'func_data': '/Users/labmanager/Resilience/derivatives/sub-01/ses-1/func/sub-01_ses-1_task-RS_run-1_bold.feat/filtered_func_data_denoised_standard.nii.gz', 'advanced': 'No', 'atlas': 'Brainnetome', 'data_space': 'Yes', 'out_space': 'MNI space', 'dir': '/Users/labmanager/JnJ_analyses/ConCorr/Atlases/Brainnetome', 'lut': '/Users/labmanager/JnJ_analyses/ConCorr/Atlases/Brainnetome/Brainnetome.lut', 'file': 'BN_Atlas_246_2mm.nii.gz', 'root': 'Brainnetome', 'max': '246.000000'}
    run_timeseries(user_info)



#========================================================================#
# Script Name: main.py                                                   #
#                                                                        #
# Description: runs ConCorr GUI and calls processes chosen           #
#                                                                        #
# Author:      Jen Burrell (March 8th, 2023)                             #
#========================================================================#

import os
import logging
from GUI import *
from timeseries import *
from corr_mat import *

def main():
    info = start_gui()

    ### --- Call Requested Processing --- ###
    if info['Processing'][0] == True:
        # - Timeseries Processing - #
        t_info = {x:info[x] for x in info if x in ('WDIR','sub_count','func_data')}
        t_gui_info = timeseries_gui()
        atlas_info = get_atlas(t_gui_info['atlas'], info['WDIR'], info['func_data'])
        run_timeseries(t_info, t_gui_info, atlas_info)
    elif info['Processing'][1] == True:
        # - Sub-level Corr Matrix  - #
        c_info = {x:info[x] for x in info if x in ('WDIR','sub_count','func_data')}
        c_gui_info = sub_corr_gui()
        atlas_info = get_atlas(c_gui_info['atlas'], info['WDIR'], info['func_data'])
        corr_mat(c_info, c_gui_info, atlas_info)
    #elif info['Processing'][2] == True:
    #    # - Grp-level Corr Matrix - #
    #    grp_corr()
    #elif info['Processing'][3] == True:
    #    # - ELA Processing - #
    #    ela()
    #elif info['Processing'][4] == True:
    #    # - CAPs Processing - #
    #    caps()

if __name__ == '__main__':
#    try:
    main()
#    except BaseException as e:
#        error_gui(e)
### ERROR NOT WORKING RIGHT



#======================================================================#
# Script Name: test_script.py                                          #
#                                                                      #
# Description: This script is used to test defined functions for bugs  #
#                                                                      #
# Author:      labmanager Burrell (March 10th, 2023)                          #
#======================================================================#
import os
import re
from timeseries import *
from GUI import *
from misc import *
from corr_mat import *
from datetime import datetime

### --- testing GUIs --- ###
#info = start_gui()
#print(info)

#info = timeseries_gui()
#print(info)
#exit()

### --- testing misc.py --- ###
info = sub_corr_gui()
print(info)
exit()

### --- testing timeseries.py --- ###
# - testing get_atlas - #
WDIR = "/Users/labmanager/Resilience"
atlas_input = "HCP-MMP1" ### TESTING
func = '/Users/labmanager/Resilience/derivatives/sub-01/ses-1/func/sub-01_ses-1_task-RS_run-1_bold.feat/filtered_func_data_denoised_standard.nii.gz'
#
#atlas_info = get_atlas(atlas_input, WDIR, func)
#atlas = "HCP-MMP1"
##a = {'dir': (f"{SDIR}/Atlases/{atlas}"), 'file': 'Schaefer2018_100Parcels_7Networks_order_FSLMNI152_2mm.nii.gz'}
#a = {'dir': (f"{SDIR}/Atlases/{atlas}")}
#pix_info = ''.join(str(subprocess.check_output(["fslinfo", f"{func}"])).split('\\t')).split('\\')
#pix_idx = [i for i, s in enumerate(pix_info) if 'npixdim1' in s]
#pix_dim = pix_info[pix_idx[0]].split("npixdim1")[1].split('.')[0]
#a['lut'] = os.path.join(a['dir'], f"{atlas}.lut")
#a['file'] = os.path.join(a['dir'], f"BN_Atlas_246_{pix_dim}mm.nii.gz")
#a['root'] = a['file'].split('.')[0]
#print(a['root'].values)
#exit()

# - testing timeseries_run() - #
t_gui_info = {'atlas': 'Provided_ROIs', 'data_space': 'Yes', 'out_space': 'MNI space', 'advanced': 'No'}
atlas_info = {'dir': '/Users/labmanager/Resilience/ROIs', 'lut': [], 'root': 'Provided_ROIs', 'file': ['Hipp.nii', 'MPFC.nii', 'DMPFC.nii', 'PCC.nii', 'sgACC.nii', 'ACC.nii', 'VMPFC.nii', 'DLPFC.nii', 'RLPFC.nii'], 'max':9}
info = {'WDIR': '/Users/labmanager/Resilience', 'sub_count': 4, 'func_data': '/Users/labmanager/Resilience/derivatives/sub-01/ses-1/func/sub-01_ses-1_task-RS_run-1_bold.feat/filtered_func_data_denoised_standard.nii.gz', 'file_names': ['filtered_func_data_denoised_standard.nii.gz', 'filtered_func_data_denoised_standard.nii.gz', 'filtered_func_data_denoised_standard.nii.gz', 'filtered_func_data_denoised_standard.nii.gz', 'filtered_func_data_denoised_standard.nii.gz', 'filtered_func_data_denoised_standard.nii.gz', 'filtered_func_data_denoised_standard.nii.gz', 'filtered_func_data_denoised_standard.nii.gz', 'filtered_func_data_denoised_standard.nii.gz', 'filtered_func_data_denoised_standard.nii.gz', 'filtered_func_data_denoised_standard.nii.gz', 'filtered_func_data_denoised_standard.nii.gz', 'filtered_func_data_denoised_standard.nii.gz', 'filtered_func_data_denoised_standard.nii.gz', 'filtered_func_data_denoised_standard.nii.gz', 'filtered_func_data_denoised_standard.nii.gz', 'filtered_func_data_denoised_standard.nii.gz', 'filtered_func_data_denoised_standard.nii.gz', 'filtered_func_data_denoised_standard.nii.gz'], 'Processing': [True, False, False, False, False]}
t_info = {x:info[x] for x in info if x in ('WDIR','sub_count','func_data')}
user_info = {'WDIR': '/Users/labmanager/Resilience', 'sub_count': 4, 'func_data': '/Users/labmanager/Resilience/derivatives/sub-01/ses-1/func/sub-01_ses-1_task-RS_run-1_bold.feat/filtered_func_data_denoised_standard.nii.gz', 'advanced': 'No', 'atlas': 'Schaefer', 'data_space': 'Yes', 'out_space': 'MNI space', 'dir': '/Users/labmanager/JnJ_analyses/ConCorr/Atlases/Schaefer', 'file': 'Schaefer2018_100Parcels_7Networks_order_FSLMNI152_2mm.nii.gz', 'root': 'Schaefer2018_100Parcels_7Networks_order', 'lut': '/Users/labmanager/JnJ_analyses/ConCorr/Atlases/Schaefer/fsleyes_lut/Schaefer2018_100Parcels_7Networks_order.lut', 'min': '0.000000', 'max': '100.000000'}
# - define variables in use - #
func_data = user_info['func_data']
atlas = user_info['atlas'] # name of atlas
atlas_lut_file = user_info['lut']
atlas_directory = user_info['dir']
parce_file = user_info['file'] # name of atlas file
parce_name = user_info['root'] # name of atlas
max = int(float(user_info['max']))
if user_info['advanced'] == 'Yes':
    user_thr = user_info['thr']
else:
    user_thr = 0.75

# - get path for all participants - #
for char in func_data:
    if char.isdigit():
        func_data = func_data.replace(char, '*')
        
dir_path_list = sorted(glob.glob(os.path.dirname(func_data))) # list of all files
file_list = {}
for i_sub in range(1, user_info['sub_count']+1):
    r = re.compile(f".*/sub-.{i_sub}/.*")
    files = list(filter(r.match, dir_path_list))
    file_list[f"{i_sub}"] = files
    
print(file_list['2'])
#run_timeseries(user_info)
exit()

# - Extras - #
#func_data = user_info['func_data']
#atlas = user_info['atlas'] # name of atlas
#atlas_lut_file = user_info['lut']
#atlas_directory = user_info['dir']
#parce_file = user_info['file'] # name of atlas file
#parce_name = user_info['root'] # name of atlas file
#roi_name = 'ACC'
#roi_name_file = 'ACC.nii'
#dir_path = '/Users/labmanager/Resilience/derivatives/sub-01/ses-1/func/sub-01_ses-1_task-RS_run-1_bold.feat'
#func_file = 'filtered_func_data_denoised.nii.gz'

# - testing sub_corr_gui - #
#gui_info = sub_corr_gui()
c_gui_info = {'atlas': 'Provided_ROIs', 'corr_coef': 'Pearson', 'partial': 'Yes', 'stars': 'Yes'}
atlas_info = {'dir': '/Users/labmanager/Resilience/ROIs', 'lut': [], 'root': 'Provided_ROIs', 'file': ['Hipp.nii', 'MPFC.nii', 'DMPFC.nii', 'PCC.nii', 'sgACC.nii', 'ACC.nii', 'VMPFC.nii', 'DLPFC.nii', 'RLPFC.nii'], 'max':9}
c_info = {x:info[x] for x in info if x in ('WDIR','sub_count','func_data')}

#sub_corr(c_info, c_gui_info, atlas_info)
user_info = {'WDIR': '/Users/labmanager/Resilience', 'sub_count': 4, 'func_data': '/Users/labmanager/Resilience/derivatives/sub-01/ses-1/func/sub-01_ses-1_task-RS_run-1_bold.feat/filtered_func_data_denoised_standard.nii.gz', 'atlas': 'Provided_ROIs', 'corr_coef': 'Pearson', 'partial': 'Yes', 'stars': 'Yes', 'dir': '/Users/labmanager/Resilience/ROIs', 'lut': [], 'root': 'Provided_ROIs', 'file': ['Hipp.nii', 'MPFC.nii', 'DMPFC.nii', 'PCC.nii', 'sgACC.nii', 'ACC.nii', 'VMPFC.nii', 'DLPFC.nii', 'RLPFC.nii'], 'max': 9}

parce_name = user_info['root'] # name of atlas
dir_path = '/Users/labmanager/Resilience/derivatives/sub-01/ses-1/func/sub-01_ses-1_task-RS_run-1_bold.feat'
d1 = f"{dir_path}/ROI_timeseries/{parce_name}_timeseries.txt"
i_sub = list(filter(lambda x: "sub-" in x, d1.split("/")))[0].split('-')[1]

task_name = re.sub(f"bold.*/ROI_timeseries/{parce_name}_timeseries.txt", "", d1) # get rid of path after run specific information
task_name = task_name[task_name.find(i_sub):].split("/")[-1]

print(task_name)




#==================================================================================#
# Script Name: ELA.py                                                              #
#                                                                                  #
# Description: lol who knows, nothing yet hopefully energy landscape analysis soon #
#                                                                                  #
# Authors:     Jen Burrell (Jan 30th, 2023)                                        #
#==================================================================================#
# To Do List:
# convert each roi timeseries to z-scores, (value - mean)/SD , where mean is the average activation level of the roi
# find average over time for each roi (empirical rate of activation)
# find averages of all pairwise products over time (empirical pairwise joint activation rate)
# fit a pairwise maximum entropy model (pMEM) * insert yoshi sound here *
#---------------------------------------#
#  Import relavent packages or install  #
#       if not done so already          #
#---------------------------------------#
import sys
import subprocess
import re
import itertools
import os
import warnings
import math
try:
    import numpy as np
except ImportError as e:
    subprocess.check_call([sys.executable, '-m', 'pip', 'install', 'numpy'])
    import numpy as np
try:
    import pandas as pd
except ImportError as e:
    subprocess.check_call([sys.executable, '-m', 'pip', 'install', 'pandas'])
    import pandas as pd
try:
    import matplotlib.pyplot as plt
    from matplotlib.patches import Patch
except ImportError as e:
    subprocess.check_call([sys.executable, '-m', 'pip', 'install', 'matplotlib.pyplot'])
    import matplotlib.pyplot as plt
    from matplotlib.patches import Patch
#---------------------------------------#
#             Get data from             #
#           Timecourse script           #
#---------------------------------------#
d1 = sys.argv[1] # data path
i_sub = sys.argv[2] # subject number
corr_out = sys.argv[3] # output path
atlas = sys.argv[4]
if sys.argv[5:]:
    atlas_lut = sys.argv[5]
task_name = re.sub(f"bold.*/ROI_timeseries/*_timeseries.txt", "", d1) # get rid of path after run specific information
task_name = task_name[task_name.find(i_sub):] # remove folders preceeding task information
task_name = task_name.split('bold', 1)[0] # only task specific information
corr_out = corr_out  + '/' # path includes subject folder
d1 = pd.read_csv(d1,sep='\t') # read in data
roi_list = list(d1.columns.values)  # get list of roi_names
if "Schaefer" in atlas: # get network names
    atlas_lut = pd.read_table(atlas_lut, sep=' ', header=None)
    net_names = atlas_lut.iloc[: , -1: ]
    net_names_raw = net_names.rename(columns={4:'Networks'})
    net_names = net_names[4].str.split('_', n=3, expand=True) # breaks string of network names
    net_names['Network'] = net_names[1] + "_" + net_names[2] # stitches together hemispheric networks
#    net_names['Network'] = net_names[2] # just networks
    net_comp = net_names[1] + "_" + net_names[2] + "_" + net_names[3]
    net_names = net_names.pop('Network') # gets network names
    d1.columns = net_names
    unq_names = pd.DataFrame(net_names.unique())

# - z-transformation - #
z_scaled = d1.copy()
for column in d1:
    z_scaled[column] = (z_scaled[column] - z_scaled[column].mean()) / z_scaled[column].std()
z_scaled = z_scaled.T
    
# - binarize - #
binary = pd.DataFrame(np.where(z_scaled>0, 1, 0), z_scaled.index, z_scaled.columns)

# - save dataframe to file - #
os.makedirs(corr_out + 'Stats/' + atlas + '/', exist_ok=True) # makes directory, or moves on
binary.to_csv(corr_out + 'Stats/' + atlas + '/' + task_name + 'binarized_timeseries.txt', index=None, header=None, sep=' ')

if "Schaefer" in atlas:
    # - average by network - #
    z_scaled_net = z_scaled.groupby(by=['Network']).mean()
   
    # - binarize - #
    binary = pd.DataFrame(np.where(z_scaled_net>0, 1, 0), z_scaled_net.index, z_scaled_net.columns)

    # - save dataframe to file - #
    os.makedirs(corr_out + 'Stats/' + atlas + '/', exist_ok=True) # makes directory, or moves on
    
    unq_names.to_csv(corr_out + 'Stats/' + atlas + '/timeseries_networks.dat', sep='\n', index=None, header=None, )
    binary.to_csv(corr_out + 'Stats/' + atlas + '/' + task_name + 'binarized_timeseries_net.txt', index=None, header=None, sep=' ')



#========================================================================#
# Script Name: corr_mat.py                                               #
#                                                                        #
# Description: Script creates correlation matrices with full and partial #
#              correlations from timecourses generated by parent script  #
#              via ConCorr                                           #
#                                                                        #
# Authors:      Jen Burrell (Mar 20th, 2023)                             #
#========================================================================#
# To Do List:
# sliding window
#    tapered and weighted edges
#    overlap between windows (1 TR to window length)
#    30 s, 60 s, 120 s, and 240 s windows

import sys
import subprocess
import re
import itertools
import os
import warnings
import math
import glob
from sklearn import metrics
import numpy as np
import pandas as pd
import seaborn as sns
import pingouin as pg
import matplotlib.pyplot as plt
from matplotlib.patches import Patch
from scipy.cluster import hierarchy
from sympy import symbols, solve
from misc import *
from itertools import repeat
from multiprocessing import Pool


#            python3 "$SDIR"/correlation_matrix.py $dir_path/ROI_timeseries/${file_root}_timeseries.txt $i_sub $corr_out $file_root $atlas_lut_file

#--------------------------------------#
#       Correlation Calculations       #
#          and Matrix Building         #
#--------------------------------------#
def corr_mat(*dicts):
    user_info = {}
    for dict in dicts:
        user_info.update(dict)
    print(user_info)
        
    func_data = user_info['func_data']
    
    # - get path for all participants - #
    for char in func_data:
        if char.isdigit():
            func_data = func_data.replace(char, '*')
            
    dir_path_list = sorted(glob.glob(os.path.dirname(func_data))) # list of all files
    
    # - multiprocessing - #
    pool = Pool() # Create a multiprocessing Pool
    pool.starmap(sub_corr, zip(dir_path_list, repeat(user_info))) # process gets masks
    # Close pool, keep kids safe
    pool.close()
    pool.join()

def sub_corr(dir_path, user_info):
    # - define variables in use - #
    WDIR = user_info['WDIR']
    corr_coef = user_info['corr_coef']
    p_adjust = user_info['p_adjust']
    partial = user_info['partial']
    atlas = user_info['atlas'] # name of chosen atlas
    parce_name = user_info['root'] # name of atlas
    atlas_lut = user_info['lut']
    
    # - load in data - #
    if not file_exists(f"{dir_path}/ROI_timeseries/{parce_name}_timeseries.txt"):
        print(f"{dir_path}/ROI_timeseries/{parce_name}_timeseries.txt")
        exit(f"Please run timeseries generation for {parce_name} and try again")
    
    d1 = f"{dir_path}/ROI_timeseries/{parce_name}_timeseries.txt"
    i_sub = list(filter(lambda x: "sub-" in x, d1.split("/")))[0].split('-')[1]
    corr_out = f"{WDIR}/ConCorr/sub-{i_sub}/"
    mkdir(corr_out)
    
    task_name = re.sub(f"bold.*/ROI_timeseries/{parce_name}_timeseries.txt", "", d1) # get rid of path after run specific information
    task_name = task_name[task_name.find(i_sub):].split("/")[-1] # remove folders preceeding task information taking only task specific information

    ### --- Read in data --- ###
    print("Reading in the data")
    d1 = pd.read_csv(d1,sep='\t') # read in data
    roi_list = list(d1.columns.values)  # get list of roi_names
    
    if "Schaefer" in atlas:
        atlas_lut = pd.read_table(atlas_lut, sep=' ', header=None)
        net_names= atlas_lut.iloc[: , -1: ]
        net_names_raw = net_names.rename(columns={4:'Networks'})
        net_names = net_names[4].str.split('_', n=3, expand=True) # breaks string of network names
        net_names['Network'] = net_names[1] + "_" + net_names[2] # stitches together hemispheric networks
    #    net_names['Network'] = net_names[2] # just networks
        net_names = net_names.pop('Network') # gets network names
        
    warnings.simplefilter(action='ignore', category=FutureWarning) # pingouin needs to update their code so ignore warnings
    if len(d1.axes[0]) < len(d1.axes[1]): # if less data points than covariates, partial correlations are invalid
        print("Partial Correlations will not be calculated, less data points than covariates")

    ### --- stats --- ###
    print("Starting stats")
    mkdir(corr_out, f"Stats/{parce_name}")
    corr1_name = corr_out + 'Stats/' + parce_name + '/' + task_name + 'full_corr_stats.csv'
    if len(d1.axes[0]) > len(d1.axes[1]) and partial == 'Yes':
        corr2_name = corr_out  + 'Stats/' + parce_name + '/' + task_name + 'partial_corr_stats.csv'
    
    ### --- sliding window (temporal decomposition) --- ###
    if user_info['slide'] == 'Yes'
        window = user_info['window']
        TR = user_info['TR']
        shift = user_info['shift'] # sliding bar from TR to window length
        edges = user_info['edges'] # rectangle, hamming (np.hamming()), tukey (scipy.signal.windows.tukey())
        if edges == 'rectangle':
            
        elif edges == 'hamming':
        elif edges == 'tukey':
    
    
    
    
    
    
    
    
    
    
    # - correlations - #
    corr1 = d1.corr() # full correlation
    warnings.simplefilter(action='ignore', category=RuntimeWarning) # dividing by zero results in error
    corr1_z_plot = np.arctanh(corr1) # fisher's z transformation for graph
    if partial == "Yes": # partial correlation
        corr2 = d1.pcorr()
        corr2_z_plot = np.arctanh(corr2) # fisher's z transformation for graph
    
    # - full correlations metrics - #
    if not file_exists(corr1_name):
        print("Computing Full Correlations")
        corr1_stats = d1.pairwise_corr(method=corr_coef, padjust=p_adjust) # stats for full correlation corrected for multiple comparison
        corr1_z = np.arctanh(corr1_stats.loc[:,"r"].values) # fisher's z transformation for table
        corr1_stats['Fishers Z'] = np.array(corr1_z)
        corr1_stats.to_csv(corr1_name, compression=None) # save full correlation stats table
    else:
        corr1_stats = pd.read_csv(corr1_name)
    # - partial correlations metrics - #
    if partial == "Yes":
        if len(d1.axes[0]) > len(d1.axes[1]) and partial == 'Yes':
            if not file_exists(corr2_name):
                p_corr_stats = pd.DataFrame()
                for ind, subset in enumerate(itertools.combinations(roi_list, len(roi_list) - 2)):
                    corr2_stats = d1.pairwise_corr(covar=list(subset), method=corr_coef)
                    p_corr_stats = pd.concat([p_corr_stats, corr2_stats], ignore_index=True) # stats for all partial comparisons
                    print("Computing partial correlation (",ind,"/",math.comb(len(roi_list), len(roi_list) - 2),")")
                reject, adj_p_val = pg.multicomp(p_corr_stats.loc[:,"p-unc"], method=p_adjust) # calculate adjusted p-values
                adj_p_val = list(adj_p_val)
                p_corr_stats['p-corr'] = np.array(adj_p_val) # add adjusted p-values to stats structure
                corr2_z = np.arctanh(p_corr_stats.loc[:,"r"].values) # fisher's z transformation for table
                p_corr_stats['Fishers Z'] = np.array(corr2_z)
                p_corr_stats.to_csv(corr2_name, compression=None) # save partial correlation stats table
            else:
                p_corr_stats = pd.read_csv(corr2_name)

    ### --- masks --- ###
    print("Making Masks")
    mask1 = np.triu(np.ones_like(corr1_z_plot, dtype=bool)) # mask for upper triangle (data appears in lower triangle)
    mask2 = np.tril(np.ones_like(corr2_z_plot, dtype=bool)) # mask for lower triangle (data appears in upper triangle)

    #---------------------------------------#
    #            Generate Figure            #
    #---------------------------------------#
    mkdir(corr_out, f"Images/{parce_name}") # makes directory
    if len(d1.axes[0]) > len(d1.axes[1]) and partial == "Yes": # if less data points than covariates, partial correlations are invalid
        if len(corr1) <= 30 and user_info['stars'] == 'Yes':
            ### --- create matrix of astriks for p-values --- ###
            star_mat_full = star_map(corr1_stats['p-corr'])
            star_mat_part = star_map_p(p_corr_stats['p-corr'])
            
            ### --- figure generation with star maps --- ###
            print("Building Figure")
            sns.set_theme(style="white")
            fig, ax = plt.subplots(figsize=(10, 10))

            sns.heatmap(corr1_z_plot, mask=mask1, cmap='Blues', vmax=1, vmin=-1,
                        square=True, linewidths=.1, cbar_kws={"shrink": .66, "pad":.02, "label": 'Full Correlations'}, ax=ax, annot=star_mat_full, fmt='') # full correlation in lower triangle
            sns.heatmap(corr2_z_plot, mask=mask2, cmap='YlOrBr', vmax=1, vmin=-1, square=True, linewidths=.1, cbar_kws={"shrink": .66, 'label': 'Partial Correlations'}, ax=ax, annot=star_mat_part, fmt='') # partial correlation in upper triangle
            ax.patch.set_facecolor('black') # diagonals
            
    #        plt.show() # uncomment to see image at generation
            plt.savefig(corr_out + 'Images/' + parce_name + '/' + task_name + 'corr-mat_z.png', bbox_inches="tight", pad_inches=0.3) # saves image folder of subject folder in specified output directory
        else:
            ### --- figure generation --- ###
                print("Building Figure")
                labels = [*range(0, len(corr1_z_plot)+1, (len(corr1_z_plot))//10)]
                sns.set_theme(style="white")
                fig, ax = plt.subplots(figsize=(10, 10))

                ax = sns.heatmap(corr1_z_plot, mask=mask1, cmap='Blues', vmax=1, vmin=-1, square=True, linewidths=0.0, rasterized=True, xticklabels=labels, yticklabels=labels, cbar_kws={"shrink": .66, "pad":.02, "label": 'Full Correlations'}) # full correlation in lower triangle
                ax = sns.heatmap(corr2_z_plot, mask=mask2, cmap='YlOrBr', vmax=1, vmin=-1, square=True, linewidths=0.0, rasterized=True, xticklabels=labels, yticklabels=labels, cbar_kws={"shrink": .66, "pad":.02, "label": 'Partial Correlations'} ) # partial correlation in upper triangle
                ax.patch.set_facecolor('black') # diagonals
                ax.set(title=f"{task_name} using {parce_name}", xlabel='Nodes', ylabel='Nodes')
                ax.set_xticks(labels, labels=labels)
                ax.set_yticks(labels, labels=labels)

    #            plt.show() # uncomment to see image at generation
                os.makedirs(corr_out + 'Images/' + parce_name + '/', exist_ok=True) # makes directory, or moves on
                plt.savefig(corr_out + 'Images/' + parce_name + '/' + task_name + 'corr-mat_z.png', bbox_inches="tight", pad_inches=0.3) # saves image folder of subject folder in specified output directory
    else:
        if len(corr1) <= 30 and user_info['stars'] == 'Yes':
            ### --- create matrix of astriks for p-values --- ###
            star_mat_full = star_map(corr1_stats['p-corr'])

            ### --- figure generation with star maps --- ###
            print("Building Figure")
            sns.set_theme(style="white")
            fig, ax = plt.subplots(figsize=(10, 10))

            sns.heatmap(corr1_z_plot, cmap="viridis", vmax=1, vmin=-1, square=True, linewidths=.1, cbar_kws={"shrink": .66, "pad":.02}, ax=ax, annot=star_mat_full, fmt='') # full correlation in lower triangle
            ax.patch.set_facecolor('black') # diagonals

    #        plt.show() # uncomment to see image at generation
            plt.savefig(corr_out + 'Images/' + parce_name + '/' + task_name + 'corr-mat_z.png', bbox_inches="tight", pad_inches=0.3) # saves image folder of subject folder in specified output directory
        else:
            ### --- figure generation --- ###
            print("Building Figure")
            labels = [*range(0, len(corr1_z_plot)+1, (len(corr1_z_plot))//10)]
            sns.set_theme(style="white")
            fig, ax = plt.subplots(figsize=(10, 10))

            ax = sns.heatmap(corr1_z_plot, cmap="viridis", vmax=1, vmin=-1, square=True, linewidths=0.0, rasterized=True, xticklabels=labels, yticklabels=labels, cbar_kws={"shrink": .66, "pad":.02})
            ax.patch.set_facecolor('black') # diagonals
            ax.set(title=f"{task_name} using {parce_name}", xlabel='Nodes', ylabel='Nodes')
            ax.set_xticks(labels, labels=labels)
            ax.set_yticks(labels, labels=labels)

    #        plt.show() # uncomment to see image at generation
            plt.savefig(corr_out + 'Images/' + parce_name + '/' + task_name + 'corr-mat_z.png', bbox_inches="tight", pad_inches=0.3) # saves image folder of subject folder in specified output directory

    if "Schaefer" in atlas and user_info['cluster'] == 'Yes':
        ### --- cluster figure generation --- ###
        print("Building Figure")
        labels = [*range(0, len(corr1_z_plot)+1, (len(corr1_z_plot))//10)]
        sns.set_theme(style="white")
        # - get colours for the unique networks present - #
        row_colors = np.random.randint(5, size=len(net_names.unique()))
        pal1 = sns.color_palette('GnBu', n_colors=len(net_names.unique())//2) # uncomment for hemispheric networks
        pal2 = sns.color_palette('BuGn', n_colors=len(net_names.unique())//2) # uncomment for hemispheric networks
        palette = pal1 + pal2 # uncomment for hemispheric networks
        #pal1 = sns.color_palette('GnBu', n_colors=len(net_names.unique())) # uncomment for whole networks
        #palette = pal1 # uncomment for whole networks
        lut = dict(zip(net_names.unique(), sns.color_palette(palette, n_colors=len(net_names.unique()))))
        row_colors = net_names.map(lut)

        # - figure - #
        g = sns.clustermap(corr1.reset_index(drop=True), row_colors=row_colors, cmap="viridis", linewidths=0.0, xticklabels=False, yticklabels=False, figsize=(12, 10))
        ax = g.ax_heatmap
        g.fig.subplots_adjust(right=0.7,top=0.96)
        g.ax_cbar.set_position((0.74, .2, .03, .4))
        ax.set(xlabel='Nodes', ylabel='Nodes')
        g.fig.suptitle(f"Clustering of networks from {task_name} using {parce_name}")
        handles = [Patch(facecolor=lut[name]) for name in lut]
        plt.legend(handles, lut, title='Networks', bbox_to_anchor=(0.91, 0.5), bbox_transform=plt.gcf().transFigure, loc='center')

        # - get the clustering for networks from dendrogram - #
        link_net = hierarchy.fcluster(g.dendrogram_col.linkage, len(net_names.unique())*2, criterion='maxclust')
        link_order = g.dendrogram_row.reordered_ind
        net = dict(sorted(zip(link_order, link_net)))
        net = pd.DataFrame.from_dict(net, orient='index')

        # - compare cluster to parce_name - #
        rand = round(metrics.rand_score(net_names, net[0]), 5)
        string = 'Rand score: ' + str(rand)
        plt.text(1, 0, string, transform=ax.transAxes)

    #    plt.show() # uncomment to see image at generation
        plt.savefig(corr_out + 'Images/' + parce_name + '/' + task_name + 'clust-mat.png', bbox_inches="tight", pad_inches=0.3) # saves image folder of subject folder in specified output directory



#***************************************#
#               Star Maps               #
#***************************************#
# - full corr - #
def star_map(p_values):
    corr_star_full = pd.Series(np.empty(len(p_values)))
    for r_ind, corr in enumerate(p_values):
        if corr <= 0.05:
            corr_star_full.loc[r_ind] = '*'
            if corr <= 0.01:
                corr_star_full.loc[r_ind] = '**'
                if corr <= 0.001:
                    corr_star_full.loc[r_ind] = '***'
        else:
            corr_star_full.loc[r_ind] = ''
    x = symbols('x')
    expr = x**2 - x - 2*len(corr_star_full)
    shape = solve(expr)[1]
    star_mat_full = np.chararray((shape, shape), itemsize=3, unicode=True)
    triu = np.triu_indices(len(star_mat_full), k = 1)
    tril = np.tril_indices(len(star_mat_full), -1)
    star_mat_full[triu] = corr_star_full
    star_mat_full[tril] = star_mat_full.T[tril]
    return star_mat_full

# - partial corr - #
def star_map_p(p_values):
    corr_star_part = pd.Series(np.empty(len(p_values)))
    for r_ind, corr in enumerate(p_values):
        if corr <= 0.05:
            corr_star_part.loc[r_ind] = '*'
            if corr <= 0.01:
                corr_star_part.loc[r_ind] = '**'
                if corr <= 0.001:
                    corr_star_part.loc[r_ind] = '***'
        else:
            corr_star_part.loc[r_ind] = ''
    corr_star_part = corr_star_part[::-1] # ROIs tested backwards compared to full correlation, so correct by flipping values
    x = symbols('x')
    expr = x**2 - x - 2*len(corr_star_part)
    shape = solve(expr)[1]
    star_mat_part = np.chararray((shape, shape), itemsize=3, unicode=True)
    triu = np.triu_indices(len(star_mat_part), k = 1)
    tril = np.tril_indices(len(star_mat_part), -1)
    star_mat_part[triu] = corr_star_part
    star_mat_part[tril] = star_mat_part.T[tril]
    return star_mat_part


if __name__ == '__main__':
    user_info = {'WDIR': '/Users/labmanager/Resilience', 'sub_count': 4, 'func_data': '/Users/labmanager/Resilience/derivatives/sub-01/ses-1/func/sub-01_ses-1_task-RS_run-1_bold.feat/filtered_func_data_denoised_standard.nii.gz', 'atlas': 'Schaefer', 'corr_coef': 'pearson', 'p_adjust': 'holm', 'partial': 'Yes', 'stars': 'Yes', 'cluster': 'Yes', 'dir': '/Users/labmanager/JnJ_analyses/ConCorr/Atlases/Schaefer', 'file': 'Schaefer2018_100Parcels_7Networks_order_FSLMNI152_2mm.nii.gz', 'root': 'Schaefer2018_100Parcels_7Networks_order', 'lut': '/Users/labmanager/JnJ_analyses/ConCorr/Atlases/Schaefer/fsleyes_lut/Schaefer2018_100Parcels_7Networks_order.lut', 'max': '100.000000'}
    corr_mat(user_info)



#========================================================================#
# Script Name: correlation_group.py                                      #
#                                                                        #
# Description: Script creates allows for the creation of different correlation     #
#              matrices with full and partial correlations from different group    #
#              connections from time courses generate by parent script   #
#              'Timecourse_JandJ.sh'                                     #
#                                                                        #
# Authors:      Jen Burrell (Dec 7th, 2022)                              #
#========================================================================#
# To Do List:

#---------------------------------------#
#  Import relavent packages or install  #
#       if not done so already          #
#---------------------------------------#
import sys
import subprocess
import re
import itertools
import os
import glob
import warnings
try:
    import numpy as np
except ImportError as e:
    subprocess.check_call([sys.executable, '-m', 'pip', 'install', 'numpy'])
    import numpy as np
try:
    import pandas as pd
except ImportError as e:
    subprocess.check_call([sys.executable, '-m', 'pip', 'install', 'pandas'])
    import pandas as pd
try:
    import seaborn as sns
except ImportError as e:
    subprocess.check_call([sys.executable, '-m', 'pip', 'install', 'seaborn'])
    import seaborn as sns
try:
    import matplotlib.pyplot as plt
except ImportError as e:
    subprocess.check_call([sys.executable, '-m', 'pip', 'install', 'matplotlib.pyplot'])
    import matplotlib.pyplot as plt
try:
    import pingouin as pg
except ImportError as e:
    subprocess.check_call([sys.executable, '-m', 'pip', 'install', 'pingouin'])
    import pingouin as pg

#---------------------------------------#
#         Get directories from          #
#           Timecourse script           #
#---------------------------------------#
derivatives = sys.argv[1] # derivatives path
corr_out = sys.argv[2] + '/' # output path
sub_list = sys.argv[3] # subject list
sub_list = sorted(sub_list.split(','))
ses_list = sys.argv[4] # session list
ses_list = sorted(ses_list.split(','))
atlas = sys.argv[5]
design = corr_out + 'design_matrix.csv'
d_mat = pd.read_csv(design, header=None) # read in design matrix

### --- Set up Stats readout --- ###
# - ONLY WORKS WITH SQUARE DATA - #
d_mat = d_mat.fillna(0)
d_list = d_mat.values.tolist()
d_list_flat = d_mat.values.flatten().tolist()
sub_list_ex = []
ses_list_ex = []
run_list = []
col_list = []
if len(ses_list)>0:
    for i_part in range(len(d_list)):
        for i_run in range(len(d_list_flat)//len(d_list)):
            run_list.append('run-' + str(i_run+1))
            sub_list_ex.append(sub_list[i_part])
            for ses_num in range(len(ses_list)):
                for i_ses in range((len(d_list_flat)//len(d_list))//len(ses_list)):
                    ses_list_ex.append(ses_list[ses_num])
    small_list = list(zip(d_list_flat, sub_list_ex, ses_list_ex, run_list))
    col_list.append(['Group','Subject','Session','Run'])
else:
    for i_part in range(len(d_list)):
        for i_run in range(len(d_list_flat)//len(d_list)):
            run_list.append('run-' + str(i_run+1))
            sub_list_ex.append(sub_list[i_part])
    small_list = list(zip(d_list_flat, sub_list_ex, run_list))
    col_list.append(['Group','Subject','Run'])

#---------------------------------------#
#          Find data from files         #
#---------------------------------------#
stats_files = []
part_files = []
full_files = []
full_r_data = []
full_z_data = []
part_r_data = []
part_z_data = []
for i_sub in sub_list:
    stats_path = corr_out + i_sub + '/Stats/' + atlas
    for result in glob.iglob(stats_path):
        for root, dirs, files in os.walk(result, topdown=False):
            for name in files:
                if name.endswith("corr_stats.csv"):
                    stats_files.append(os.path.join(root, name)) # get all files with path
stats_files.sort() # sort so runs and participants are in the right order
full_files = list(filter(lambda substring: "full" in substring, stats_files))
part_files = list(filter(lambda substring: "partial" in substring, stats_files))

### --- Set up Stats readout --- ###
# - ONLY WORKS WITH SQUARE DATA - #
f_data_list =[]
med_list = []
for file in full_files:
    full_data = pd.read_csv(file, sep=',') # read in data
    roi_names = full_data.loc[:,"X"] # both lists are incomplete
    roi_names_2 = full_data.loc[:,"Y"]
    roi_pairs = [(roi_names[i], roi_names_2[i]) for i in range(0, len(roi_names))]
    roi_pairs = ['-'.join(sub_list) for sub_list in roi_pairs]
    full_z_data = full_data.loc[:,"Fishers Z"].values
    full_r_data = full_data.loc[:,"r"].values
    f_data_list.append(list(zip(roi_pairs,full_r_data,full_z_data)))
flat_list = [item for sublist in f_data_list for item in sublist]
col_list.append(['Roi_Pairs','Pearsons_R',"Fishers_Z"])
col_list = [item for sublist in col_list for item in sublist]
for i_data in range(len(small_list)):
    for i_roi in range(len(roi_pairs)):
        med_list.append(small_list[i_data])
big_list = list(zip(med_list,flat_list))
big_flat_list = [item for sublist in big_list for item in sublist]
really_flat_list = [item for sublist in big_flat_list for item in sublist]
less_flat_list = list(itertools.zip_longest(*[iter(really_flat_list)]*len(col_list)))
if len(ses_list)>0:
    f_stats = pd.DataFrame(less_flat_list, columns=col_list)
else:
    f_stats = pd.DataFrame(less_flat_list, columns=col_list)
os.makedirs(corr_out + 'Stats/' + atlas + '/', exist_ok=True) # makes directory, or moves on
f_stats.to_csv(corr_out  + 'Stats/' + atlas + '/Full_group_stats_{atlas}.csv', compression=None, index=False) # save partial correlation stats table

#---------------------------------------#
#             Design Matrix             #
#---------------------------------------#
pos_list = [(d_mat[d_mat[col].eq(1)].index[i], d_mat.columns.get_loc(col)) for col in d_mat.columns for i in range(len(d_mat[d_mat[col].eq(1)].index))]
neg_list = [(d_mat[d_mat[col].eq(-1)].index[i], d_mat.columns.get_loc(col)) for col in d_mat.columns for i in range(len(d_mat[d_mat[col].eq(-1)].index))]
pos_df = pd.DataFrame(pos_list, columns =['Part', 'Run'])
neg_df = pd.DataFrame(neg_list, columns =['Part', 'Run'])

f_files_dic = {}
p_files_dic = {}
f_pos_files = []
p_pos_files = []
f_neg_files = []
p_neg_files = []
for i_part in sub_list:
    f_files_dic[i_part] = list(filter(lambda x: i_part in x, full_files))
    f_files_df = pd.DataFrame.from_dict(f_files_dic, orient='index')
    p_files_dic[i_part] = list(filter(lambda x: i_part in x, part_files))
    p_files_df = pd.DataFrame.from_dict(p_files_dic, orient='index')

if len(part_files)>0:
    for i in range(len(pos_df)):
        f_pos_files.append(f_files_df.iloc[pos_list[i]])
        p_pos_files.append(p_files_df.iloc[pos_list[i]])
    for i in range(len(neg_df)):
        f_neg_files.append(f_files_df.iloc[neg_list[i]])
        p_neg_files.append(p_files_df.iloc[neg_list[i]])
else:
    for i in range(len(pos_df)):
        f_pos_files.append(f_files_df.iloc[pos_list[i]])
    for i in range(len(neg_df)):
        f_neg_files.append(f_files_df.iloc[neg_list[i]])

task_list = []
z_data = []
### --- full data from files --- ###
# - positive data - #
task_list = []
z_data = []
for file in f_pos_files:
    task_name = re.sub("_full_corr_stats.csv", "", file) # get rid of path after run specific information
    task_name = task_name.split("/")[-1] # only run specific information
    task_list.append(task_name) # all of the tasks
    full_data = pd.read_csv(file, sep=',') # read in data
    roi_names = full_data.loc[:,"X"] # both lists are incomplete
    roi_names_2 = full_data.loc[:,"Y"]
    z_data.append(full_data.loc[:,"Fishers Z"].values)

f_pos_z_data = {}
z_data = np.array(z_data) # array of arrays holding all of the z data from files
f_pos_z_data = pd.DataFrame(data=z_data.T, columns=[task_list]) # turns array into dataframe with columns for each file

# - negative data - #
if len(neg_list)>0:
    task_list = []
    z_data = []
    for file in f_neg_files:
        task_name = re.sub("_full_corr_stats.csv", "", file) # get rid of path after run specific information
        task_name = task_name.split("/")[-1] # only run specific information
        task_list.append(task_name) # all of the tasks
        full_data = pd.read_csv(file, sep=',') # read in data
        roi_names = full_data.loc[:,"X"] # both lists are incomplete
        roi_names_2 = full_data.loc[:,"Y"]
        z_data.append(full_data.loc[:,"Fishers Z"].values)

    f_neg_z_data = {}
    z_data = np.array(z_data) # array of arrays holding all of the z data from files
    f_neg_z_data = pd.DataFrame(data=z_data.T, columns=[task_list]) # turns array into dataframe with columns for each file

### --- partial data from files --- ###
if len(part_files)>0:
    # - positive data - #
    task_list = []
    z_data = []
    for file in p_pos_files:
        task_name = re.sub("_partial_corr_stats.csv", "", file) # get rid of path after run specific information
        task_name = task_name.split("/")[-1] # only run specific information
        task_list.append(task_name) # all of the tasks
        part_data = pd.read_csv(file, sep=',') # read in data
        z_data.append(part_data.loc[:,"Fishers Z"].values)
        
    p_pos_z_data = {}
    z_data = np.array(z_data) # array of arrays holding all of the z data from files
    p_pos_z_data = pd.DataFrame(data=z_data.T, columns=[task_list]) # turns array into dataframe with columns for each file
    
    # - negative data - #
    if len(neg_list)>0:
        task_list = []
        z_data = []
        for file in p_neg_files:
            task_name = re.sub("_partial_corr_stats.csv", "", file) # get rid of path after run specific information
            task_name = task_name.split("/")[-1] # only run specific information
            task_list.append(task_name) # all of the tasks
            part_data = pd.read_csv(file, sep=',') # read in data
            z_data.append(part_data.loc[:,"Fishers Z"].values)
        
        p_neg_z_data = {}
        z_data = np.array(z_data) # array of arrays holding all of the z data from files
        if len(z_data)>0:
            p_neg_z_data = pd.DataFrame(data=z_data.T, columns=[task_list]) # turns array into dataframe with columns for each file
    
    ### --- calculate data --- ###
    f_pos_avg = f_pos_z_data.mean(axis=1)
    p_pos_avg = p_pos_z_data.mean(axis=1)
    if len(neg_list)>0:
        f_neg_avg = f_neg_z_data.mean(axis=1)
        p_neg_avg = p_neg_z_data.mean(axis=1)
    
        pos_neg_full = f_pos_avg - f_neg_avg
        pos_neg_part = p_pos_avg - p_neg_avg
    else:
        pos_neg_full = f_pos_avg
        pos_neg_part = p_pos_avg
        
    roi_names = pd.concat([roi_names, roi_names_2]) # concatenate lists
    roi_names = list(dict.fromkeys(roi_names)) # make the names into dictionary key terms then create list, there by removing doubles of any entry
    
    ### --- full matrix --- ###
    group_mat_full = np.zeros((len(roi_names),len(roi_names)))
    triu = np.triu_indices(len(roi_names), k = 1)
    tril = np.tril_indices(len(roi_names), -1)
    index = np.triu_indices_from(group_mat_full, k = 1)
    group_mat_full[triu] = pos_neg_full[:]
    group_mat_full[tril] = group_mat_full.T[tril]

    ### --- partial matrix --- ###
    group_mat_part = np.zeros((len(roi_names),len(roi_names)))
    triu = np.triu_indices(len(roi_names), k = 1)
    tril = np.tril_indices(len(roi_names), -1)
    index = np.triu_indices_from(group_mat_part, k = 1)
    group_mat_part[triu] = pos_neg_part[:]
    group_mat_part[tril] = group_mat_part.T[tril]
else:
    ### --- calculate data --- ###
    f_pos_avg = f_pos_z_data.mean(axis=1)
    if len(neg_list)>0:
        f_neg_avg = f_neg_z_data.mean(axis=1)
        
        pos_neg_full = f_pos_avg - f_neg_avg
    else:
        pos_neg_full = f_pos_avg
    
    roi_names = pd.concat([roi_names, roi_names_2]) # concatenate lists
    roi_names = list(dict.fromkeys(roi_names)) # make the names into dictionary key terms then create list, there by removing doubles of any entry
    
    ### --- full matrix --- ###
    group_mat_full = np.zeros((len(roi_names),len(roi_names)))
    triu = np.triu_indices(len(roi_names), k = 1)
    tril = np.tril_indices(len(roi_names), -1)
    index = np.triu_indices_from(group_mat_full, k = 1)
    group_mat_full[triu] = pos_neg_full[:]
    group_mat_full[tril] = group_mat_full.T[tril]
    
### --- masks --- ###
mask1 = np.triu(np.ones_like(group_mat_full, dtype=bool)) # mask for upper triangle (data appears in lower triangle)
mask2 = np.tril(np.ones_like(group_mat_full, dtype=bool)) # mask for lower triangle (data appears in upper triangle)

### --- Figure Generation --- ###
if len(part_files)>0:
    if len(group_mat_full) <= 30:
        sns.set_theme(style="white")
        fig, ax = plt.subplots(figsize=(11, 9))
        ax.set(title=f"Group mean Fisher's z-scores subtraction matrix")

        sns.heatmap(group_mat_full, mask=mask1, cmap='Blues', square=True, linewidths=.0, xticklabels = roi_names, yticklabels = roi_names, cbar_kws={"shrink": .66, "pad":.02, 'label': 'Full Correlations'}) # full correlation in lower triangle
        sns.heatmap(group_mat_part, mask=mask2, cmap='YlOrBr',square=True, linewidths=.0, xticklabels = roi_names, yticklabels = roi_names, cbar_kws={"shrink": .66, 'label': 'Partial Correlations'}) # partial correlation in upper triangle
        ax.patch.set_facecolor('black') # diagonals

#        plt.show() # uncomment to see image at generation
        os.makedirs(corr_out + 'Group_Images/', exist_ok=True) # makes directory, or moves on
        plt.savefig(corr_out + 'Group_Images/' f'avg-corr-mat_subtract_{atlas}.png') # saves image folder of subject folder in
    else:
        sns.set_theme(style="white")
        labels = [*range(0, len(group_mat_full)+1, (len(group_mat_full))//10)]
        fig, ax = plt.subplots(figsize=(11, 9))
        ax.set(title=f"Group mean Fisher's z-scores subtraction matrix")

        sns.heatmap(group_mat_full, mask=mask1, cmap='Blues', square=True, linewidths=.0, xticklabels=labels, yticklabels=labels, cbar_kws={"shrink": .66, "pad":.02, 'label': 'Full Correlations'}) # full correlation in lower triangle
        sns.heatmap(group_mat_part, mask=mask2, cmap='YlOrBr',square=True, linewidths=.0, xticklabels=labels, yticklabels=labels, cbar_kws={"shrink": .66, 'label': 'Partial Correlations'}) # partial correlation in upper triangle
        ax.patch.set_facecolor('black') # diagonals
        ax.set_xticks(labels, labels=labels)
        ax.set_yticks(labels, labels=labels)

#        plt.show() # uncomment to see image at generation
        os.makedirs(corr_out + 'Group_Images/', exist_ok=True) # makes directory, or moves on
        plt.savefig(corr_out + 'Group_Images/' f'avg-corr-mat_subtract_{atlas}.png') # saves image folder of subject folder in
else:
    sns.set_theme(style="white")
    labels = [*range(0, len(group_mat_full)+1, (len(group_mat_full))//10)]
    fig, ax = plt.subplots(figsize=(11, 9))
    
    sns.heatmap(group_mat_full, cmap="viridis", square=True, rasterized=True, xticklabels=labels, yticklabels=labels, cbar_kws={"shrink": .66, "pad":.02, 'label': 'Full Correlations'}) # full correlation in lower triangle
    ax.patch.set_facecolor('black') # diagonals
    ax.set(title="Group mean Fisher's z-scores subtraction matrix", xlabel='Nodes', ylabel='Nodes')
    ax.set_xticks(labels, labels=labels)
    ax.set_yticks(labels, labels=labels)
    
#    plt.show() # uncomment to see image at generation
    os.makedirs(corr_out + 'Group_Images/', exist_ok=True) # makes directory, or moves on
    plt.savefig(corr_out + 'Group_Images/' f'avg-corr-mat_subtract_{atlas}.png') # saves image folder of subject folder in



#===============================================================================#
# Script Name: setup.py                                                         #
#                                                                               #
# Description: ensures all toolboxes necessary are downloaded                   #
#                                                                               #
# Author:      Jen Burrell (March 8th, 2023)                                    #
#===============================================================================#
import os.path as op
from setuptools import setup
from setuptools import find_namespace_packages
from setuptools import Command

# - setup.py file's home directory - #
basedir = op.dirname(__file__)

# - Get Version - #
version = {}
with open(op.join(basedir, "src/ConCorr/version.py")) as f:
    for line in f:
        if line.startswith('__version__'):
            exec(line, version)
            break
version = version['__version__']

# - Readme - #
with open(op.join(basedir, 'README.rst'), 'rt') as f:
    readme = f.read()

# - Dependencies are listed in requirements.txt - #
#with open(op.join(basedir, 'requirements.txt'), 'rt') as f:
#    install_requires = [l.strip() for l in f.readlines()]

# - set it up! - #
setup(
    name='ConCorr',
    version=version,
    description='A small fMRI functional connectivity toolbox',
    long_description=readme,
    long_description_content_type='text/x-rst',
    url='https://github.com/jenburrell/ConCorr',
    author='Jen Burrell',
    author_email='jenbur@psych.ubc.ca',
    python_requires='>=3.10',
    
    package_dir={"":"src"},
    packages=find_namespace_packages(where='src'),
    
    classifiers=[
    'Programming Language :: Python :: 3.10',
    'Operating System :: OS Independent',
    'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    
    install_requires=[
        'numpy>=1',
        'scipy>=1.9.3',
        'pandas>=1.5.0',
        'seaborn>=0.12.1',
        'matplotlib>=3.6.0',
        'pingouin>=0.5.3',
        'scikit-learn>=1.0.2',
        'fastcluster>=1.2.6',
    ],
    extras_require={
        "dev": [
            "pytest>=3.7",
        ],
    },
)



### --- BEFORE PUBLISHING --- ###
# check-manifest --create
# git add MANIFEST.in

# python setup.py sdist # to do source distribution
# tar tzf dist/ConCorr-0.0.1.tar.gz # should show lots of files see pic

### --- TO PUBLISH --- ###
# python setup.py bdist_wheel sdist
# python install twine
# python upload dist/*




### --- CAPS Analysis prep --- ###
if [[ "$caps_analysis" == "yes" ]] ; then
    ### --- Set up folders --- ###
    if [ ! -d "$dir_path/CAPs" ] ; then # check to see if ROI mask folder exists for this participant's run, if not make it
        mkdir -p $dir_path/CAPs
    fi
    if [ ! "$(ls -A $dir_path/CAPs)" ] ; then # checks to see if folder is empty, if it is then fill it with files
        fslsplit $dir_path/$func_data $dir_path/CAPs/vol_ -t # CAPs toolbox requires 3D files not 4D
        gunzip $dir_path/CAPs/* # and must be unziped
    fi
        fi



#========================================================================#
# Script Name: GUI.py                                                    #
#                                                                        #
# Description: creates ConCorr GUIs                                  #
#                                                                        #
# Author:      Jen Burrell (March 8th, 2023)                             #
#========================================================================#

import PySimpleGUI as sg
import os
import filecmp
import logging
from misc import *
from datetime import datetime
# Add your new theme colors and settings
#sg.LOOK_AND_FEEL_TABLE['MyCreatedTheme'] = {'BACKGROUND': '#5B6858',
#                                        'TEXT': '#F6FFD5',
#                                        'INPUT': '#BA6D4A',
#                                        'TEXT_INPUT': '#000000',
#                                        'SCROLL': '#99CC99',
#                                        'BUTTON': ('#F6FFD5', '#424D5B'),
#                                        'PROGRESS': ('#D1826B', '#CC8019'),
#                                        'BORDER': 1, 'SLIDER_DEPTH': 0,
#                                        'PROGRESS_DEPTH': 0, }

sg.theme_global('GreenTan')   # Add a touch of color
### --- run starting GUI --- ###
def start_gui():
    info = {}
    center_column = [[sg.Text("Welcome to ConCorr")]]
    buttons = [
        [sg.Button('Load')],
        [sg.Button('Go'), sg.Button('Exit')]
        ]
    tab1_layout = [
        [sg.Text("Select your project working directory:"), sg.In(enable_events=True, key='-WDIR-'),sg.FolderBrowse('Select')],
        [sg.Text("Select a functional file that will be used in analysis:"), sg.Input(enable_events=True, key='-func_data-'), sg.FilesBrowse('Select')],
        [sg.Text("", enable_events=True, key='-oh_func-' ), sg.Button('Manual Selection', enable_events=True, visible=False, key='-oh_func_butt-')],
        [sg.VPush()],
        [sg.Text("Subjects found: "), sg.Text("None", enable_events=True, key='-sub_list-')],
        [sg.Text("Total # of Files found: "), sg.Text("None", enable_events=True, key='-file_list-')],
        [sg.Text("", enable_events=True, key='-move_on-')],
        ]
    tab2_layout = [
        [sg.Text("Select processing options below:")],
        [sg.Checkbox("Timeseries Generation", key='-timeseries-'), sg.Push(), sg.Button("info", key='-ts_butt-')],
        [sg.Text("Subject-Level processing:")],
        [sg.Checkbox("Correlation Matrix Generation", key='-sub_mat-'), sg.Push(), sg.Button("info", key='-sub_butt-')],
        [sg.Text("Group-Level processing:")],
        [sg.Checkbox("Correlation Matrix Generation", key='-grp_mat-'), sg.Push(), sg.Button("info", key='-grp_butt-')],
        [sg.Checkbox("Energy Landscape Analysis Toolbox Preprocessing", key='-ela_pre-'), sg.Push(), sg.Button("info", key='-ela_butt-')],
        [sg.Checkbox("Co-Activation Pattern Analysis Toolbox Preprocessing", key='-caps_pre-'), sg.Push(), sg.Button("info", key='-cap_butt-')],
        ]
    layout = [
        [sg.Push(), sg.Column(center_column,element_justification='c'), sg.Push()],
        [sg.TabGroup([[
            sg.Tab('DATA', tab1_layout, key= '-tab1-', tooltip='Where you specify your data'),
            sg.Tab('Processing', tab2_layout, key= '-tab2-', tooltip='Where you specify what happens to your data'),
            ]],enable_events=True)],
        [sg.VPush()],
        [sg.Push(), sg.Column(buttons,element_justification='c'), sg.Push()],
        ]
    window = sg.Window('ConCorr', layout, resizable=True, enable_close_attempted_event=True)
    while True:
        event, values = window.read()
        if event == sg.WIN_CLOSED:
            break
        elif event == 'Exit' or event == '-WINDOW CLOSE ATTEMPTED-':
            exit("User exited ConCorr")
        elif event == '-WDIR-':
            WDIR = values['-WDIR-']
            try:
                sub_list = [i for i in os.listdir(f'{WDIR}/derivatives') if i.startswith("sub-")]
                sub_count = len(sub_list)
            except:
                sub_count = 0
            window['-sub_list-'].update(sub_count)
            info['WDIR'] = WDIR
            info['sub_count'] = sub_count
        elif event == '-func_data-':
            func_data = values['-func_data-']
            func_file = os.path.basename(func_data)
            file_names = [file for _, _, f in os.walk(f'{WDIR}/derivatives') for file in f if file.lower().endswith(func_file)]
            if len(file_names) < sub_count:
                window['-oh_func-'].update("Data could not be found please manually select data \nHint: your data has to be named the same across participants to be found automatically")
                window['-oh_func_butt-'].update(visible=True)
                window['Go'].update(disabled=True)
            file_count = len(file_names)
            window['-file_list-'].update(file_count)
            window['-move_on-'].update("If above information is correct, move on to Processing tab :)")
            info['func_data'] = values['-func_data-']
            info['file_names'] = file_names
        elif event == '-oh_func_butt-':
            by_hand_dict = by_hand()
            info['file_names'] = by_hand_dict['file_names']
            info['func_data'] = by_hand_dict['func_data']
            file_count = len(info['file_names'])
            window['-file_list-'].update(file_count)
            window['Go'].update(disabled=False)
        elif event == 'Load':
            load_info = load_gui()
            info = {**info, **load_info}
            WDIR = info['WDIR']
            try:
                sub_list = [i for i in os.listdir(f'{WDIR}/derivatives') if i.startswith("sub-")]
                sub_count = len(sub_list)
            except:
                sub_count = 'oops'
            file_count = len(info['file_names'])
            window['-sub_list-'].update(sub_count)
            window['-file_list-'].update(file_count)
            window['-move_on-'].update("If above information is correct, move on to Processing tab or 'Go' to use previous processing methods :)")
        elif event == '-ts_butt-':
            sg.popup_no_border("Timeseries Generation \n\nThis module will generate a timeseries based on the ROIs specified. \nYou are able to use your own ROI's by having a folder named 'ROIs' in your Project folder. \nYou can also choose from the available atlases included with ConCorr. \nAtlases include: Brainnetome, HCP-MMP1, HMAT, Juelich, and Schaefer \n\nMore information about atlases and Timeseries Generation can be found in the User Guide :)")
        elif event == '-sub_butt-':
            sg.popup_no_border("Subject-Level Correlation Matrix Generation \n\nThis module will generate correlation matrices for each functional file using timeseries from specified ROIs. Parameters such as correlation coeffiecient can be modified. \n\nMore information about Subject-Level Correlation Matrix Generation can be found in the User Guide :)")
        elif event == '-grp_butt-':
            sg.popup_no_border("Group-Level Correlation Matrix Generation \n\nThis module will generate averaged correlation matrices using timeseries from specified ROIs and a Design Matrix. Parameters such as correlation coeffiecient and group can be modified. \n\nMore information about Group-Level Correlation Matrix Generation can be found in the User Guide :)")
        elif event == '-ela_butt-':
            sg.popup_no_border("Energy Landscape Analysis Toolbox Preprocessing \n\nThis module will generate the necessary file types to run subject data through the Energy Landscape Analysis Toolbox (ELAT; Ezaki et al. PPhilosophical Transactions. Series A, Mathematical, Physical, and Engineering Sciences, 375(2096), 20160287 (2017)). \n\nMore information about Energy Landscape Analysis Toolbox Preprocessing can be found in the User Guide :)")
        elif event == '-cap_butt-':
            sg.popup_no_border("Co-Activation Pattern Analysis Toolbox Preprocessing \n\nThis module will generate the necessary file types to run subject data through the TbCAPs: A toolbox for co-activation pattern analysis (Bolton et al. NeuroImage, 211, 116621. (2020)). \n\nMore information about Energy Landscape Analysis Toolbox Preprocessing can be found in the User Guide :)")
        elif event == 'Go':
            info['Processing'] = [values['-timeseries-'], values['-sub_mat-'], values['-grp_mat-'], values['-ela_pre-'], values['-caps_pre-']]
            if not True in info['Processing']:
                info['Processing'] = load_info['Processing']
                if not True in info['Processing']:
                    sg.popup_no_border("No analysis has been choosen :(, \nif you want the program to do something go to the 'Processing' tab and select the analyses of your choice")
                    continue
            # - SAVE TEXT FILE OF INFO - #
            ConCorr_folder = f"{WDIR}/ConCorr/info"
            save_fld = mkdir(ConCorr_folder)
            file_list = [os.path.join(ConCorr_folder, f) for f in os.listdir(ConCorr_folder)]
            current = datetime.now().strftime("%d-%m-%Y_%H-%M-%S")
            mkfile(f"{ConCorr_folder}/info_parts-{info['sub_count']}_files-{file_count}_{current}.txt", info)
            for file in file_list:
                samesies = filecmp.cmp(file, f"{ConCorr_folder}/info_parts-{info['sub_count']}_files-{file_count}_{current}.txt")
                if samesies:
                    os.remove(file)
            window.close()
    window.close()
    return info
    

def by_hand():
    ### --- Manual selection of Data --- ###
    info = {}
    all_files = []
    center_column = [[sg.Text("Manual Data Selection")]]
    file_list_column = [
        [sg.Text("Select a folder that contains data \nThen click all of the data files you want to include \nOnce all data appears in the list on the right, click 'Go'")],
        [sg.Text("Folder"), sg.In(size=(30, 1), enable_events=True, key='-FOLDER-'), sg.FolderBrowse()],
        [sg.Listbox(
            values=[],
            enable_events=True,
            size=(50, 20),
            key='-FILE_LIST-'
            )],
        ]
    file_viewer_column = [
        [sg.Text("Ensure all data files are in list before continuing", size=(50, 1))],
        [sg.Text("Files: ", size=(70, 3), key='-TOUT-')],
        [sg.Multiline(size=(70, 30), key='-TEXT-')]
        ]
    buttons = [[sg.Button('Go'), sg.Button('Exit')]]
    layout = [
        [sg.Push(), sg.Column(center_column,element_justification='c'), sg.Push()],
        [sg.Column(file_list_column), sg.VSeperator(), sg.Column(file_viewer_column)],
        [sg.Push(), sg.Column(buttons,element_justification='c'), sg.Push()],
        ]
    window = sg.Window('ConCorr', layout, resizable=True)
    while True:
        event, values = window.read()
        if event == sg.WIN_CLOSED or event == 'Exit':
            break
        elif event == '-FOLDER-':
            folder_location = values['-FOLDER-']
            try:
                files = os.listdir(folder_location)
            except:
                files = []
            file_names = [
                file for file in files
                if os.path.isfile(os.path.join(folder_location, file))
                and file.lower().endswith((".gz", ".nii"))
                ]
            window['-FILE_LIST-'].update(file_names)
        elif event == '-FILE_LIST-' and len(values['-FILE_LIST-']) > 0:
            if values['-FILE_LIST-'][0] not in all_files:
                all_files.append(values['-FILE_LIST-'][0])
            else:
                all_files.remove(values['-FILE_LIST-'][0])
            window['-TEXT-'].update(all_files)
        elif event == 'Go':
            info['file_names'] = all_files
#            func_data = os.path.join(folder_location, all_files[0]))
            info['func_data'] = os.path.join(folder_location, all_files[0])
            window.close()
    window.close()
    return info
    
def load_gui():
    ### --- Load info files --- ###
    info = {}
    center_column = [[sg.Text("Load info files")]]
    buttons = [[sg.Button('Load'), sg.Button('Cancel')]]
    content = [[sg.Text("Select info file to load: "), sg.In(size=(30, 1), enable_events=True, key='-load-'), sg.FileBrowse()]]
    layout = [
        [sg.Push(), sg.Column(center_column,element_justification='c'), sg.Push()],
        [content],
        [sg.VPush()],
        [sg.Push(), sg.Column(buttons,element_justification='c'), sg.Push()],
        ]
    window = sg.Window('ConCorr', layout, resizable=True, enable_close_attempted_event=True)
    while True:
        event, values = window.read()
        if event == sg.WIN_CLOSED or event == 'Cancel':
            break
        elif event == '-WINDOW CLOSE ATTEMPTED-':
            exit("User exited ConCorr")
        elif event == 'Load':
            load_file = values['-load-']
            if file_exists(load_file):
                info = load(load_file)
                info = info[0]
                window.close()
            else:
                sg.popup_no_border("Load file not found :( \nPlease try again or select data through data selection window")
    window.close()
    return info
            
        
def timeseries_gui():
    ### --- Present options for timeseries generation --- ###
    info = {}
    center_column = [[sg.Text("Timeseries Generation")]]
    buttons = [
        [sg.Button('Advanced')],
        [sg.Button('Go'), sg.Button('Exit')]
        ]
    content = [
        [sg.Text("How would you like to define your ROIs"), sg.Push(), sg.OptionMenu(['Brainnetome', 'HCP-MMP1', 'HMAT', 'Juelich', 'Schaefer', 'Provide your own ROIs'], default_value="...", key='-atlas-')],
        [sg.Text("Is your data in MNI space?"), sg.Push(), sg.OptionMenu(['Yes', 'No'], default_value="...", key='-data_space-')],
        [sg.Text("", enable_events=True, key='-out_text-' ), sg.Push(), sg.OptionMenu(['MNI space', 'Subject space'], default_value='MNI space', visible=False, key='-out_space-')]
        ]
    layout = [
        [sg.Push(), sg.Column(center_column,element_justification='c'), sg.Push()],
        [content],
        [sg.Push(), sg.Column(buttons,element_justification='c'), sg.Push()],
        ]
    info['advanced'] = 'No'
    window = sg.Window('ConCorr', layout, resizable=True, enable_close_attempted_event=True)
    while True:
        event, values = window.read()
        if event == sg.WIN_CLOSED:
            break
        elif event == 'Exit' or event == '-WINDOW CLOSE ATTEMPTED-':
            exit("User exited ConCorr")
        elif event == 'Go':
            info['atlas'] = values['-atlas-']
            info['data_space'] = values['-data_space-']
            if '...' in info['atlas'] or '...' in info['atlas']:
                sg.popup_no_border("Selections required for processing")
                continue
            if info['data_space'] == 'No' and not 'out_space' in info:
                buttons_3 = [[sg.Button('Go'), sg.Button('Cancel')]]
                layout_3 = [
                    [sg.Text("Would you like to generate the timeseries in MNI space or in subject space?"), sg.Push(), sg.OptionMenu(['MNI space', 'Subject space'], default_value='...', key='-out_space-')],
                    [sg.VPush()],
                    [sg.VPush()],
                    [sg.Push(), sg.Column(buttons_3,element_justification='c'), sg.Push()],
                    ]
                window_3 = sg.Window("ConCorr",layout_3, resizable=True)
                while True:
                    event, values = window_3.read()
                    if event == 'Cancel' or event == sg.WIN_CLOSED:
                        break
                    if event == 'Go':
                        info['out_space'] = values['-out_space-']
                        window_3.close()
                window_3.close()
                continue
            else: info['out_space'] = 'MNI space'
            if info['atlas'] == 'Provide your own ROIs':
                info['atlas'] = 'Provided_ROIs'
                sg.popup_no_border("Provided ROIs must be in folder named 'ROIs' in your project working directory")
            if info['data_space'] == 'Yes' and info['out_space'] == 'Subject space':
                sg.popup_no_border("Data cannot be moved back into subject space, analysis will be done in MNI space")
                info['out_space'] = 'MNI space'
            window.close()
        elif event == 'Advanced':
            buttons_2 = [[sg.Button('Go'), sg.Button('Cancel')]]
            layout_2 = [
                [sg.Text("Set 'fslmaths' threshold for (0-1)"), sg.Input(enable_events=True, key='-thr-')],
                [sg.VPush()],
                [sg.VPush()],
                [sg.Push(), sg.Column(buttons_2,element_justification='c'), sg.Push()],
                ]
            window_2 = sg.Window("Advanced", layout_2, resizable=True)
            while True:
                event, values = window_2.read()
                if event == 'Cancel' or event == sg.WIN_CLOSED:
                    break
                if event == 'Go':
                    if not values['-thr-'] == '':
                        info['advanced'] = 'Yes'
                        info['thr'] = values['-thr-']
                    window_2.close()
            window_2.close()
    window.close()
    return info

def reg_gui():
    ### --- Throw error for registration -- ###
    info = {}
    text = "We weren't able to find the files necessary to normalize your data to MNI space,"
    text_2 = "you can normalize your subjects then try again, or choose to complete analysis in subject space"
    center_column = [
        [sg.Text("OH NO!")],
        [sg.Push()],
        [sg.Text(text)],
        [sg.Text(text_2)],
        [sg.Text(":(")],
        ]
    buttons = [[sg.Button("Fine I'll do it myself"), sg.Button("Subject Space!")]]
    layout = [
        [sg.Push(), sg.Column(center_column,element_justification='c'), sg.Push()],
        [sg.VPush()],
        [sg.Push(), sg.Column(buttons,element_justification='c'), sg.Push()],
        ]
    window = sg.Window('ConCorr', layout, resizable=True, enable_close_attempted_event=True)
    while True:
        event, values = window.read()
        if event == sg.WIN_CLOSED:
            break
        elif event == 'Exit' or event == '-WINDOW CLOSE ATTEMPTED-':
            exit("User exited ConCorr")
        elif event == "Fine I'll do it myself":
            exit("User is going to do it themselves")
        elif event == "Subject Space!":
            info['out_space'] = "Subject space"
            window.close()
    window.close()
    return info

def sub_corr_gui():
    ### --- Present options for correlation matrix generation -- ###
    info = {}
    center_column = [[sg.Text("Subject-level Correlation Matrices")]]
    buttons = [[sg.Button('Go'), sg.Button('Exit')]]
    content = [
        [sg.Text("Select processing options below:")],[
        sg.Text("What atlas would you like to use?"), sg.Push(), sg.OptionMenu(['Brainnetome', 'HCP-MMP1', 'HMAT', 'Juelich', 'Schaefer', 'Provide your own ROIs'], default_value="...", key='-atlas-')],
        [sg.Text("Choose your correlation coefficent"), sg.Push(), sg.OptionMenu(['pearson', 'kendall', 'spearman'], default_value="pearson", key='-corr_coef-')],
        [sg.Text("Choose the method for testing and adjustment of pvalues"), sg.Push(), sg.OptionMenu(['none', 'bonf', 'sidak', 'holm', 'fdr_bh', 'fdr_by'], default_value='holm', key='-p_adjust-')],
        [sg.Text("Include Partial Correlations"), sg.Push(), sg.OptionMenu(['Yes', 'No'], default_value="Yes", key='-partial-')],
        [sg.Text("Include significance asterisk"), sg.Push(), sg.OptionMenu(['Yes', 'No'], default_value="Yes", key='-stars-')]
        ]
    layout = [
        [sg.Push(), sg.Column(center_column,element_justification='c'), sg.Push()],
        [sg.Push()],
        [content],
        [sg.VPush()],
        [sg.Push(), sg.Column(buttons,element_justification='c'), sg.Push()],
        ]
    window = sg.Window('ConCorr', layout, resizable=True, enable_close_attempted_event=True)
    while True:
        event, values = window.read()
        if event == sg.WIN_CLOSED:
            break
        elif event == 'Exit' or event == '-WINDOW CLOSE ATTEMPTED-':
            exit("User exited ConCorr")
        elif event == 'Go':
            info['atlas'] = values['-atlas-']
            info['corr_coef'] = values['-corr_coef-']
            info['p_adjust'] = values['-p_adjust-']
            info['partial'] = values['-partial-']
            info['stars'] = values['-stars-']
            if info['atlas'] == 'Provide your own ROIs':
                info['atlas'] = 'Provided_ROIs'
            elif info['atlas'] == 'Schaefer':
                layout_2 = [
                    [sg.Text("Include a cluster matrix for Schaefer Networks?"), sg.Push(), sg.OptionMenu(['Yes', 'No'], default_value="Yes", key='-cluster-')],
                    [sg.Button("Ok"),sg.Button("Cancel")]]
                window_2 = sg.Window('Cluster Map?', layout_2, resizable=True, enable_close_attempted_event=True)
                while True:
                    event, values = window_2.read()
                    if event == sg.WIN_CLOSED:
                        break
                    if event == '-WINDOW CLOSE ATTEMPTED-':
                        exit("User exited ConCorr")
                    if event in 'Cancel':
                        info['cluster'] = 'No'
                        window_2.close()
                    if event == 'Ok':
                        info['cluster'] = values['-cluster-']
                        window_2.close()
                window_2.close()
            window.close()
    window.close()
    return info

def error_gui(error):
#        logging.error('Failed to do something: ' + str(e))
    ### --- Display Errors to user --- ###
    info = {}
    center_column = [[sg.Text("OH NO! \n  :(")]]
    buttons = [[sg.Button('Exit')]]
    content = [
        [sg.Text("The following error occurred:")],
        [sg.Text(error)],
        ]
    layout = [
        [sg.Push(), sg.Column(center_column,element_justification='c'), sg.Push()],
        [sg.Push()],
        [content],
        [sg.VPush()],
        [sg.Push(), sg.Column(buttons,element_justification='c'), sg.Push()],
        ]
    window = sg.Window('Error :(', layout, resizable=True, enable_close_attempted_event=True)
    while True:
        event, values = window.read()
        if event == sg.WIN_CLOSED:
            break
        elif event == 'Exit' or event == '-WINDOW CLOSE ATTEMPTED-':
            exit("User exited ConCorr")
        elif event == 'Go':
            info['atlas'] = values['-atlas-']
            if info['atlas'] == 'Provide your own ROIs':
                info['atlas'] = 'Provided_ROIs'
            info['corr_coef'] = values['-corr_coef-']
            info['p_adjust'] = values['-p_adjust-']
            info['partial'] = values['-partial-']
            info['stars'] = values['-stars-']
            window.close()
    window.close()
    return info





#========================================================================#
# Script Name: misc.py                                                   #
#                                                                        #
# Description: This script defines general functions that will to be     #
#              used by ConCorr                                       #
#                                                                        #
# Author:      Jen Burrell (March 10th, 2023)                            #
#========================================================================#
import os.path
import subprocess
import json
import shutil

# - make a directory - #
def mkdir(path,extension=''):
    path = os.path.join(path, extension)
    if not os.path.exists(path):
        os.makedirs(path)

# - check if file exists - #
def file_exists(filepath):
    return os.path.isfile(filepath)

# - make a file - #
def mkfile(name, content={}, *more):
    data = [content]
    for d in more:
        data.append(d)
    with open(name, 'w') as f:
        json.dump(data, f)
        
# - load file - #
def load(name):
    with open(name, "r") as f:
        info = json.load(f)
    return info

## - stop annoying warnings - #
#def fxn():
#    warnings.warn("deprecated", DeprecationWarning)

